import logging
from functools import update_wrapper, wraps

import click
import yaml
from bitshares import BitShares
from bitshares.instance import set_shared_bitshares_instance

log = logging.getLogger(__name__)


def common_options(func):
    @click.option('-d', '--debug', default=False, is_flag=True, help='enable debug output')
    @click.option(
        '-c', '--config', type=click.File('r'), default='./config.yml', help='specify custom path for config file'
    )
    @click.pass_context
    @wraps(func)
    def wrapper(ctx, *args, **kwargs):
        ctx.obj = {}  # stub for uptick
        ctx.config = yaml.safe_load(kwargs.pop("config"))

        # create logger
        debug = kwargs.pop("debug")
        if debug is True:
            log.setLevel(logging.DEBUG)
        else:
            log.setLevel(logging.INFO)
        handler = logging.StreamHandler()
        formatter = logging.Formatter("%(asctime)s %(levelname)s: %(message)s")
        handler.setFormatter(formatter)
        log.addHandler(handler)
        ctx.log = log

        return func(*args, **kwargs)

    return wrapper


def chain(func):
    @click.pass_context
    def new_func(ctx, *args, **kwargs):
        ctx.bitshares = BitShares(ctx.config['node_bts'], expiration=60)
        set_shared_bitshares_instance(ctx.bitshares)
        return ctx.invoke(func, *args, **kwargs)

    return update_wrapper(new_func, func)



import secrets
import string
from typing import Any, Dict, List, Optional

from bitshares.market import Market
from bitsharesbase.account import PasswordKey


def generate_password(size: int = 53, chars: str = string.ascii_letters + string.digits) -> str:
    """Generate random word with letters and digits."""
    return ''.join(secrets.choice(chars) for x in range(size))


def get_keys_from_password(
    account_name: str, password: str, prefix: str, key_types: Optional[List[str]] = None,
) -> Dict[str, str]:
    """Generates public/private keys from a password."""
    if not key_types:
        key_types = ['active', 'owner', 'memo']

    keys = {}
    for key_type in key_types:
        # PasswordKey object
        passkey = PasswordKey(account_name, password, role=key_type)

        privkey = passkey.get_private_key()
        print('{} private: {}'.format(key_type, str(privkey)))  # we need explicit str() conversion!

        # pubkey with default prefix GPH
        pubkey = passkey.get_public_key()

        # pubkey with correct prefix
        keys[key_type] = format(pubkey, prefix)
        print('{} public: {}\n'.format(key_type, keys[key_type]))

    return keys


def convert_asset(blockchain_instance: Any, from_value: float, from_asset: str, to_asset: str) -> float:
    """Converts asset to another based on the latest market value.

    :param blockchain_instance: graphene-compatible blockchain instance
    :param float from_value: Amount of the input asset
    :param string from_asset: Symbol of the input asset
    :param string to_asset: Symbol of the output asset
    :return: float Asset converted to another asset as float value
    """
    market = Market('{}/{}'.format(from_asset, to_asset), bitshares_instance=blockchain_instance)
    ticker = market.ticker()
    latest_price = ticker.get('latest', {}).get('price', None)
    precision = market['base']['precision']

    return round((from_value * latest_price), precision)


def transform_asset(
    blockchain_instance: Any, sum_balances: Dict[str, float], from_asset: str, to_asset: str
) -> Dict[str, float]:
    """In sum_balances dict, convert one asset into another.

    :param blockchain_instance: graphene-compatible blockchain instance
    :param dict sum_balances: dict with balances
    :param str from_asset: asset to convert from
    :param str to_asset: destination asset
    """
    if from_asset in sum_balances:
        amount = convert_asset(blockchain_instance, sum_balances[from_asset], from_asset, to_asset)
        sum_balances[from_asset] = 0
        if to_asset in sum_balances:
            sum_balances[to_asset] += amount
        else:
            sum_balances[to_asset] = amount
    return sum_balances



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bitsharesscripts']

package_data = \
{'': ['*']}

install_requires = \
['bitshares>=0.4.0,<0.5.0', 'click>=7.1.1,<8.0.0', 'uptick>=0.2.4,<0.3.0']

setup_kwargs = {
    'name': 'bitsharesscripts',
    'version': '2.0.0',
    'description': 'A set of scripts for BitShares',
    'long_description': "bitshares-scripts\n=================\n\nThis is a small scripts collection for doing various stuff in Bitshares\n\n* `balances_as_btc.py` - Summarize all assets on all accounts and show BTC value\n* `cancel_all_orders.py` - Cancel all orders on the specified account\n* `create_account.py` - Create new account with random password and providing all keys to stdout\n* `get_account.py` - Display account object\n* `get_worker.py` - Display worker object\n* `get_asset.py` - Display asset object\n* `get_balance.py` - Display account balances\n* `get_balance_multi.py` - Display summarized balance of multiple accounts\n* `get_feeds.py` - Show price feeds for specified asset\n* `get_keys.py` - Generate private and public keys from account name and password\n* `get_op_id.py` - Display operation id numbers and corresponding name\n* `get_witness.py` - Display witness object\n* `update_keys.py` - Change account keys using generated random password and providing all keys to stdout\n\n**Note:** some scripts are sending transactions, you need to add private active keys for your accounts via `uptick addkey`\n\nInstallation via poetry\n-----------------------\n\n1. Make sure you have installed required packages: `apt-get install gcc make libssl-dev`\n2. Install [poetry](https://python-poetry.org/docs/)\n3. Run `poetry install` to install the dependencies\n4. Copy `common.yml.example` to `common.yml` and change variables according to your needs\n5. Now you're ready to run scripts:\n\n\n```\npoetry shell\n./script.py\n```\n\nReusable parts\n--------------\n\nReusable parts are being moved slowly into `bitsharesscripts` package namespace. It can be installed via `pip install bitsharesscripts`.\n",
    'author': 'Vladimir Kamarzin',
    'author_email': 'vvk@vvk.pp.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bitfag/bitshares-scripts',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
